import h5py
import numpy as np
import cv2
import math
import json
import sys
from scipy import interpolate
from zebrazoom.code.getImage.getForegroundImage import getForegroundImage
from zebrazoom.code.getImage.headEmbededFrame import headEmbededFrame
from scipy.interpolate import UnivariateSpline
from numpy import linspace

def initialiseDistance2(distance, boundary):
  TotalBPts   = len(boundary)
  distance[0] = 0
  for i in range(1, TotalBPts+1):
    if (i == TotalBPts):
      Pt = boundary[0][0]
    else:
      Pt = boundary[i][0]
    AvantPt = boundary[i-1][0]
    Dx = AvantPt[0] - Pt[0]
    Dy = AvantPt[1] - Pt[1]
    if i:
      distance[i] = distance[i-1] + math.sqrt(Dx*Dx + Dy*Dy)
    else:
      distance[i] = math.sqrt(Dx*Dx + Dy*Dy)
  return [distance[TotalBPts], distance]

  
def PointDot(VecA, VecB):
  return (VecA[0])*(VecB[0]) + (VecA[1])*(VecB[1])


def calculateJuge(indice, distance, max):
  juge = 0
  if distance[indice] < max - distance[indice]:
    juge = (max-2*distance[indice])/max
  else:
    juge = (2*distance[indice]-max)/max
  return juge


def calculateJuge2(indice, distance, bord1, bord2, nb):
  
  dist  = 0
  dist2 = 0
  dist3 = 0
  dist4 = 0
  mindist = 10000000000
  
  if indice < bord1:
    dist  = distance[bord1] - distance[indice]
    dist2 = distance[indice] + (distance[nb] - distance[bord1])
  else:
    dist  = distance[indice] - distance[bord1]
    dist2 = distance[bord1] + ( distance[nb] - distance[indice] )
  
  if indice < bord2:
    dist3 = distance[bord2] - distance[indice]
    dist4 = distance[indice] + ( distance[nb] - distance[bord2] )
  else:
    dist3 = distance[indice] - distance[bord2]
    dist4 = distance[bord2] + ( distance[nb] - distance[indice] )
  
  if dist < mindist:
    mindist = dist

  if dist2 < mindist:
    mindist = dist2

  if dist3 < mindist:
    mindist = dist3

  if dist4 < mindist:
    mindist = dist4
  
  return mindist

  
def smoothTail(points, nbTailPoints):
  
  y = points[0]
  x = linspace(0, 1, len(y))
  
  if len(x) > 3:    
  
    s = UnivariateSpline(x, y, s=10)
    xs = linspace(0, 1, nbTailPoints)
    newX = s(xs)

    y = points[1]
    x = linspace(0, 1, len(y))
    s = UnivariateSpline(x, y, s=10)
    xs = linspace(0, 1, nbTailPoints)
    newY = s(xs)
    
  else:
  
    newX = x
    newY = y
  
  return [newX, newY]



from sklearn.svm import SVC
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.cluster import KMeans
from sklearn.mixture import GaussianMixture
from zebrazoom.dataAnalysis.dataanalysis.visualizeClusters import visualizeClusters
import matplotlib.pyplot as plt
from tkinter import simpledialog
import tkinter as tk
import numpy as np
import pickle
import shutil
import json
import os

def prepareForActiveLearning(proportions, sortedRepresentativeBouts, outputFolderResult, nbCluster, pca_result, dfParam, sortedRepresentativeBoutsIndex, tailAngles):
  
  outputFolderResult2 = os.path.join(outputFolderResult, 'activeLearning')
  if os.path.exists(outputFolderResult2):
    shutil.rmtree(outputFolderResult2)
  os.mkdir(outputFolderResult2)
  
  optimalLength = 12
  nbIterations  = 5
  
  # Plotting bouts one by one for each cluster and writing bout indices in txt file
  outF = open(os.path.join(outputFolderResult2, 'boutIndices.txt'), "w")
  for classed in range(0, len(proportions[0])):
    indices = sortedRepresentativeBouts[classed].index
    beginningMiddleEnd = False
    if beginningMiddleEnd:
      for j in range(0, 3):
        ind2size = optimalLength if len(indices) >= 3*optimalLength else max(1, int(len(indices) / 3))
        if j == 0:
          indices2 = indices[0:ind2size]
        elif j == 1:
          indices2 = indices[int(len(indices)/2) - int(ind2size/2) : int(len(indices)/2) + int(ind2size/2)]
        else:
          indices2 = indices[len(indices) - ind2size : len(indices)]
        # Plot
        fig, tabAx3 = plt.subplots(4, 3, figsize=(22.9, 8.8))
        for k in range(0, len(indices2)):
          tailAnglestab = dfParam.loc[indices2[k], tailAngles].tolist()
          tabAx3[int(k / 3), k % 3].plot(tailAnglestab, 'b', label = str(indices2[k]))
          tabAx3[int(k / 3), k % 3].legend()
          tabAx3[int(k / 3), k % 3].title.set_text(str(indices2[k]))
        region = 'mostRepresentative' if j == 0 else 'inBetween' if j == 1 else 'leastRepresentative'
        plt.savefig(os.path.join(outputFolderResult2, 'cluster' + str(classed + 1) + '_' + region + '.png'))
        # Saving indices in txt
        line = 'cluster' + str(classed) + '_' + region + ' = [' + ', '.join([str(ind) for ind in indices2]) + "]\n"
        outF.write(line)
    else:
      for j in range(0, nbIterations):
        ind2size = optimalLength if len(indices) >= 3*optimalLength else max(1, int(len(indices) / 3))
        if (j+1)*ind2size < len(indices):
          indices2 = indices[j*ind2size:(j+1)*ind2size]
          # Plot
          fig, tabAx3 = plt.subplots(4, 3, figsize=(22.9, 8.8))
          for k in range(0, len(indices2)):
            tailAnglestab = dfParam.loc[indices2[k], tailAngles].tolist()
            tabAx3[int(k / 3), k % 3].plot(tailAnglestab, 'b', label = str(indices2[k]))
            tabAx3[int(k / 3), k % 3].legend()
            tabAx3[int(k / 3), k % 3].title.set_text(str(indices2[k]))
          region = 'mostRepresentative'
          plt.savefig(os.path.join(outputFolderResult2, 'cluster' + str(classed + 1) + '_' + region + '_' + str(j*ind2size) + '_' + str((j+1)*ind2size) + '.png'))
          # Saving indices in txt
          line = 'cluster' + str(classed) + '_' + region + '_' + str(j*ind2size) + '_' + str((j+1)*ind2size) + ' = [' + ', '.join([str(ind) for ind in indices2]) + "]\n"
          outF.write(line)
      for j in range(0, nbIterations):
        ind2size = optimalLength if len(indices) >= 3*optimalLength else max(1, int(len(indices) / 3))
        if len(indices)-(j+1)*ind2size > 0:
          indices2 = indices[len(indices)-(j+1)*ind2size:len(indices)-j*ind2size]
          # Plot
          fig, tabAx3 = plt.subplots(4, 3, figsize=(22.9, 8.8))
          for k in range(0, len(indices2)):
            tailAnglestab = dfParam.loc[indices2[k], tailAngles].tolist()
            tabAx3[int(k / 3), k % 3].plot(tailAnglestab, 'b', label = str(indices2[k]))
            tabAx3[int(k / 3), k % 3].legend()
            tabAx3[int(k / 3), k % 3].title.set_text(str(indices2[k]))
          region = 'leastRepresentative'
          plt.savefig(os.path.join(outputFolderResult2, 'cluster' + str(classed + 1) + '_' + region + '_' + str(len(indices)-(j+1)*ind2size) + '_' + str(len(indices)-j*ind2size) + '.png'))
          # Saving indices in txt
          line = 'cluster' + str(classed) + '_' + region + '_' + str(len(indices)-(j+1)*ind2size) + '_' + str(len(indices)-j*ind2size) + ' = [' + ', '.join([str(ind) for ind in indices2]) + "]\n"
          outF.write(line)
  outF.close()
  
  pickle.dump({'pca_result': pca_result.tolist(), 'dfParam': dfParam}, open(os.path.join(outputFolderResult2, 'boutParameters.pkl'), 'wb'))


def activeLearning(modelUsed, nbConditions, nbCluster, outputFolderResult, N_QUERIES, manualClassicationPath):
  
  from modAL.models import ActiveLearner
  
  nbFramesTakenIntoAccount = 24
  scaleGraphs   = 1
  showFigures   = 0
  
  with open(os.path.join(outputFolderResult, 'boutParameters.pkl'), 'rb') as pickle_file:
    data = pickle.load(pickle_file)
  with open(manualClassicationPath, 'rb') as classification_file:
    manualClassifications = json.load(classification_file)
  
  pca_result = np.array(data['pca_result'])
  dfParam    = data['dfParam']

  y_train    = np.array([])
  ind_train  = []
  for ind in manualClassifications:
    classIndIndices = manualClassifications[ind]
    ind_train       = ind_train + classIndIndices
    y_train         = np.concatenate((y_train, np.array([int(ind) for i in range(0, len(classIndIndices))])))
  X_train = pca_result[ind_train, :]
  
  X_pool   = np.delete(pca_result, ind_train, axis=0)
  ind_pool = np.delete(np.array([ind for ind in range(0, len(pca_result))]), ind_train, axis=0)
  tailAngles = ['tailAngles' + str(i) for i in range(1, 25)]
  tailAngles_pool = np.array(dfParam.loc[ind_pool, tailAngles])
  
  # Learning (and predicting) first model
  
  if modelUsed == 'KNeighborsClassifier':
    model = KNeighborsClassifier(n_neighbors = nbCluster)
  elif modelUsed == 'SVC':
    model = SVC()
  elif modelUsed == 'GaussianNB':
    model = GaussianNB()
  elif modelUsed == 'KMeans':
    model = KMeans(n_clusters = nbCluster)
  elif modelUsed == 'GaussianMixture':
    model = GaussianMixture(n_components = nbCluster)
  else:
    model = KMeans(n_clusters = nbCluster)
  
  learner = ActiveLearner(estimator=model, X_training=X_train, y_training=y_train)
  
  predictions = learner.predict(pca_result)
  
  print("Prediction after first training:")
  print(predictions)
  
  # Actual active learning start
  
  def winput(title, sentence):
    root = tk.Tk()
    root.withdraw()
    y = simpledialog.askinteger(title, sentence)
    root.destroy()
    return y
  
  for index in range(N_QUERIES):
    
    query_index, query_instance = learner.query(X_pool)
    
    plt.plot(tailAngles_pool[query_index][0])
    plt.show()
    
    userInput = winput("Cluster Number", "Enter the cluster number of this bout:")
    
    X, y = X_pool[query_index].reshape(1, -1), np.array([userInput])
    
    learner.teach(X=X, y=y)
    
    # Remove the queried instance from the unlabeled pool. # Add tail angle remove
    X_pool = np.delete(X_pool, query_index, axis=0)
    tailAngles_pool = np.delete(tailAngles_pool, query_index, axis=0)
  
  # See new results
  
  predictions = learner.predict(pca_result)
  
  print("Prediction after second training:")
  print(predictions)
  
  outputFolderResult3 = os.path.join(outputFolderResult, 'activeLearnOutput' + str(1))
  if os.path.exists(outputFolderResult3):
    shutil.rmtree(outputFolderResult3)
  os.mkdir(outputFolderResult3)
  
  predictedProb = []
  
  visualizeClusters(dfParam, predictions, predictedProb, modelUsed, nbConditions, nbCluster, nbFramesTakenIntoAccount, scaleGraphs, showFigures, outputFolderResult3, 0, 1)
  
  


import numpy as np
from scipy.interpolate import UnivariateSpline
import math
from numpy.linalg import norm

def getInstaHorizontalDisplacement(curbout, nbFramesTakenIntoAccount):

  fin = curbout["BoutEnd"] - curbout["BoutStart"] + 1

  if fin > 2:
    HeadX = curbout["HeadX"].copy()
    HeadY = curbout["HeadY"].copy()
    heading  = curbout["Heading"].copy()
    
    instHorizDisp = [0]*fin
    for i in range(0, len(instHorizDisp)-1):
      p1 = np.array((HeadX[i], HeadY[i]))
      p2 = np.array((HeadX[i] + math.cos(heading[i]), HeadY[i] + math.sin(heading[i])))
      p3 = np.array((HeadX[i+1], HeadY[i+1]))
      instHorizDisp[i] = norm(np.cross(p2-p1, p1-p3))/norm(p2-p1)
    
    instHorizDisp = np.append(instHorizDisp, instHorizDisp[len(instHorizDisp)-1])
    
  else:
    instHorizDisp = [0]*fin
    
  instHorizDisp2  = [0]*nbFramesTakenIntoAccount
  
  sizeTabNotZero = min(fin, nbFramesTakenIntoAccount)
  instHorizDisp2[0:sizeTabNotZero] = instHorizDisp[0:sizeTabNotZero]

  return instHorizDisp2



from zebrazoom.code.dataPostProcessing.createPandasDataFrameOfParameters import createPandasDataFrameOfParameters
import pandas as pd
import json
import os

def generatePklDataFileForVideo(excelFileName, ZZoutputLocation, frameStepForDistanceCalculation, forcePandasRecreation=0):
  
  # Generate .pkl data file inside the result video folder if it doesn't already exist
  excelFile = pd.read_excel(excelFileName)
  
  for videoId in range(0, len(excelFile)):
    # If it exists, retrives the frameStepForDistanceCalculationUsed parameter previously used to calculate the distance travelled.
    if os.path.exists(os.path.join(os.path.join(ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id']), 'parametersUsedForCalculation.json')):
      with open(os.path.join(os.path.join(ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id']), 'parametersUsedForCalculation.json')) as json_file:
        parametersUsedForCalculation = json.load(json_file)
    else:
      parametersUsedForCalculation = {}
    
    # Generates the .pkl data file if it doesn't already exist OR if a frameStepForDistanceCalculationUsed different than before is requested or if a videoFPS or videoPixelSize different than before is used
    if not(os.path.exists(os.path.join(os.path.join(ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id']), excelFile.loc[videoId, 'trial_id'] + '.pkl'))) or not(len(parametersUsedForCalculation)) or int(frameStepForDistanceCalculation) != int(parametersUsedForCalculation['frameStepForDistanceCalculation']) or float(excelFile.loc[videoId, 'fq']) != float(parametersUsedForCalculation['videoFPS']) or float(excelFile.loc[videoId, 'pixelsize']) != float(parametersUsedForCalculation['videoPixelSize']):
      
      print("Generating pkl datafile for video " + excelFile.loc[videoId, 'trial_id'] + " , because:")
      if not(os.path.exists(os.path.join(os.path.join(ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id']), excelFile.loc[videoId, 'trial_id'] + '.pkl'))):
        print("the path", os.path.join(os.path.join(ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'], excelFile.loc[videoId, 'trial_id']), excelFile.loc[videoId, 'trial_id'] + '.pkl'), "does not exist")
      if not(len(parametersUsedForCalculation)):
        print("len(parametersUsedForCalculation) is equal to", len(parametersUsedForCalculation))
      else:
        if int(frameStepForDistanceCalculation) != int(parametersUsedForCalculation['frameStepForDistanceCalculation']):
          print("int(frameStepForDistanceCalculation)=", int(frameStepForDistanceCalculation), "and: int(parametersUsedForCalculation['frameStepForDistanceCalculation']=", int(parametersUsedForCalculation['frameStepForDistanceCalculation']))
        if float(excelFile.loc[videoId, 'fq']) != float(parametersUsedForCalculation['videoFPS']):
          print("float(excelFile.loc[videoId, 'fq'])", float(excelFile.loc[videoId, 'fq']), "float(parametersUsedForCalculation['videoFPS']):", float(parametersUsedForCalculation['videoFPS']))
        if float(excelFile.loc[videoId, 'pixelsize']) != float(parametersUsedForCalculation['videoFPS']):
          print("float(excelFile.loc[videoId, 'pixelsize'])", float(excelFile.loc[videoId, 'pixelsize']), "float(parametersUsedForCalculation['videoPixelSize'])", float(parametersUsedForCalculation['videoPixelSize']))
      
      videoName      = excelFile.loc[videoId, 'trial_id']
      videoExtension = os.path.splitext(os.path.split(excelFileName)[1])[1]
      hyperparameters = {}
      hyperparameters["nbWells"] = len(excelFile.loc[0, 'condition'][1:-1].split(','))
      hyperparameters["frameStepForDistanceCalculation"] = frameStepForDistanceCalculation
      hyperparameters["videoFPS"]       = float(excelFile.loc[videoId, 'fq'])
      hyperparameters["videoPixelSize"] = float(excelFile.loc[videoId, 'pixelsize'])
      createPandasDataFrameOfParameters(hyperparameters, videoName, videoExtension, ZZoutputLocation if excelFile.loc[videoId, 'path'] == "defaultZZoutputFolder" else excelFile.loc[videoId, 'path'])



from zebrazoom.code.findWells import findWells
from zebrazoom.code.getBackground import getBackground
from zebrazoom.code.getImage.getForegroundImage import getForegroundImage
from zebrazoom.code.trackingFolder.tracking import tracking
from zebrazoom.code.extractParameters import extractParameters
from zebrazoom.code.createSuperStruct import createSuperStruct
from zebrazoom.code.createValidationVideo import createValidationVideo
from zebrazoom.code.getHyperparameters import getHyperparameters

from sklearn.preprocessing import normalize
import multiprocessing as mp
from multiprocessing import Process
import shutil

import csv

import numpy as np
import os

from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.getTailTipManual import findTailTipByUserInput
from zebrazoom.code.trackingFolder.tailTrackingFunctionsFolder.getTailTipManual import findHeadPositionByUserInput
from zebrazoom.code.getImage.headEmbededFrame import headEmbededFrame

def getTailExtremityFirstFrame(pathToVideo, videoName, videoExt, configFile, argv):
  
  videoName = videoName + '.' + videoExt
  
  videoPath = os.path.join(pathToVideo, videoName)

  # Getting hyperparameters
  [hyperparameters, config] = getHyperparameters(configFile, videoName, videoPath, argv)
  
  frameNumber = hyperparameters["firstFrame"]
  
  [frame, thresh1] = headEmbededFrame(videoPath, frameNumber, hyperparameters)
  
  if hyperparameters["accentuateFrameForManualTailExtremityFind"]:
    quartileChose = 0.01
    lowVal  = int(np.quantile(frame, quartileChose))
    highVal = int(np.quantile(frame, 1 - quartileChose))
    frame[frame < lowVal]  = lowVal
    frame[frame > highVal] = highVal
    frame = frame - lowVal
    mult  = np.max(frame)
    frame = frame * (255/mult)
    frame = frame.astype(int)
    frame = (frame / np.linalg.norm(frame))*255

  if hyperparameters["findHeadPositionByUserInput"]:
    with open(os.path.join(pathToVideo, videoName + 'HP.csv'), mode='w') as f:
      writer = csv.writer(f, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
      writer.writerow(findHeadPositionByUserInput(frame, frameNumber, videoPath))

  with open(os.path.join(pathToVideo, videoName + '.csv'), mode='w') as f:
      writer = csv.writer(f, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)
      writer.writerow(findTailTipByUserInput(frame, frameNumber, videoPath, hyperparameters))



import cv2
import cvui
import numpy as np
import math
import pickle
import os

import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
from zebrazoom.code.trackingFolder.headTrackingHeadingCalculationFolder.headTrackingHeadingCalculation import headTrackingHeadingCalculation
from zebrazoom.code.trackingFolder.tailTracking import tailTracking
from zebrazoom.code.trackingFolder.debugTracking import debugTracking

def simpleOptimalValueSearch(PtClosest, contour, unitVector, lenX, lenY):
  
  factor = 0
  dist = 1
  maxDist = 0
  indMax = 0
  testCenter = PtClosest + factor * unitVector
  while (dist > 0) and (factor < 20) and (testCenter[0] >= 0) and (testCenter[1] >= 0) and (testCenter[0] < lenX) and (testCenter[1] < lenY):
    factor = factor + 1
    testCenter = PtClosest + factor * unitVector
    testCenter = testCenter.astype(int)
    dist = cv2.pointPolygonTest(contour, (testCenter[0], testCenter[1]), True)
    if dist > maxDist:
      maxDist = dist
      indMax  = factor
  
  testCenter = PtClosest + indMax * unitVector
  testCenter = testCenter.astype(int)
  
  return testCenter

def reajustCenterOfMassIfNecessary(contour, x, y, lenX, lenY):
  inside = cv2.pointPolygonTest(contour, (x, y), True)
  if inside < 0:
    
    minDist = 100000000000000
    indMin  = 0
    for i in range(0, len(contour)):
      Pt = contour[i][0]
      dist = math.sqrt((Pt[0] - x)**2 + (Pt[1] - y)**2)
      if dist < minDist:
        minDist = dist
        indMin  = i
    PtClosest = contour[indMin][0]
    unitVector = np.array([PtClosest[0] - x, PtClosest[1] - y])
    unitVectorLength = math.sqrt(unitVector[0]**2 + unitVector[1]**2)
    unitVector[0] = unitVector[0] / unitVectorLength
    unitVector[1] = unitVector[1] / unitVectorLength
    if False:
      factor = 5
      testCenter = PtClosest + factor * unitVector
      testCenter = testCenter.astype(int)
      while (cv2.pointPolygonTest(contour, (testCenter[0], testCenter[1]), True) <= 0) and (factor > 1):
        factor = factor - 1
        testCenter = PtClosest + factor * unitVector
    else:
      testCenter = simpleOptimalValueSearch(PtClosest, contour, unitVector, lenX, lenY)
    
    x = testCenter[0]
    y = testCenter[1]
  
  return [x, y]
  
def fillWhiteHoles(frame):
  
  frameBeforeWhiteFill = frame.copy()
  im_floodfill = frame.copy()
  h, w = frame.shape[:2]
  mask = np.zeros((h+2, w+2), np.uint8)
  cv2.floodFill(im_floodfill, mask, (0,0), 255);
  im_floodfill_inv = cv2.bitwise_not(im_floodfill)
  frame = frame | im_floodfill_inv
  
  if cv2.countNonZero(frame) > (len(frame) * len(frame[0])) * 0.95:
    frame = frameBeforeWhiteFill
    print("BACK TO IMAGE BEFORE WHITE FILL")
  
  return frame
  
def erodeThenAddWhiteBorders(frame, kernel):
  frame = cv2.erode(frame, kernel, iterations=1)
  frame[0,:] = 255
  frame[len(frame)-1,:] = 255
  frame[:,0] = 255
  frame[:,len(frame[0])-1] = 255
  return frame

def erodeThenDilateThenAddWhiteBorders(frame, kernel, nbOfIterations):
  frame = cv2.erode(frame, kernel, iterations=nbOfIterations)
  frame = cv2.dilate(frame, kernel, iterations=nbOfIterations)
  frame[0,:] = 255
  frame[len(frame)-1,:] = 255
  frame[:,0] = 255
  frame[:,len(frame[0])-1] = 255
  return frame

def addWhiteBorders(frame):
  frame[0,:] = 255
  frame[len(frame)-1,:] = 255
  frame[:,0] = 255
  frame[:,len(frame[0])-1] = 255
  return frame

def fishTailTrackingDifficultBackground(videoPath, wellNumber, wellPositions, hyperparameters, videoName):
  
  chooseValueForAnimalBodyArea = -1 #600 # -1
  showInitialVideo = True # This is not used anymore, should be removed soon
  iterativelyErodeEachImage = False
  showFramesForDebugging = hyperparameters["debugTracking"]
  dist2Threshold = 400
  historyLength  = 1000
  reduceImageResolutionPercentage = hyperparameters["reduceImageResolutionPercentage"]
  
  pathToVideo = os.path.split(videoPath)[0]
  
  kernel  = np.ones((3, 3), np.uint8)
  ROIHalfDiam = -1
  
  class CustomError(Exception):
    pass

  cap = zzVideoReading.VideoCapture(videoPath)
  if (cap.isOpened()== False): 
    print("Error opening video stream or file")
  frame_width  = int(cap.get(3))
  frame_height = int(cap.get(4))
  
  firstFrame = hyperparameters["firstFrame"]
  lastFrame  = hyperparameters["lastFrame"]
  nbTailPoints = hyperparameters["nbTailPoints"]
  
  trackingHeadTailAllAnimals = np.zeros((hyperparameters["nbAnimalsPerWell"], lastFrame-firstFrame+1, nbTailPoints, 2))
  trackingHeadingAllAnimals  = np.zeros((hyperparameters["nbAnimalsPerWell"], lastFrame-firstFrame+1))
  trackingEyesAllAnimals     = 0
  trackingProbabilityOfGoodDetection = 0
  
  fgbg = cv2.createBackgroundSubtractorKNN(dist2Threshold = dist2Threshold, history = historyLength)
  
  if os.path.exists(os.path.join(pathToVideo, videoName + '_BackgroundKNN_' + str(historyLength) + '.pkl')):
    print("Background already pre-calculated")
    with open(os.path.join(pathToVideo, videoName + '_BackgroundKNN_' + str(historyLength) + '.pkl'), 'rb') as handle:
      listOfFramesToSave = pickle.load(handle)
    for i in range(0, len(listOfFramesToSave)):
      print("calculating background, currently at frame:", i)
      frame = listOfFramesToSave[i]
      fgmask = fgbg.apply(frame)
  else:
    print("Background calculation starting")
    
    listOfFramesToSave = []
    framesToKeep = [i for i in range(0, lastFrame - 1, int((lastFrame - 1) / historyLength if (lastFrame - 1) / historyLength >= 1 else 1))]
    cap.set(1, 0)
    for i in range(0, lastFrame):
      ret, frame = cap.read()
      if i in framesToKeep and ret:
        print(i, lastFrame)
        frame = cv2.resize(frame, (int(frame_width * reduceImageResolutionPercentage), int(frame_height * reduceImageResolutionPercentage)), interpolation = cv2.INTER_AREA)
        listOfFramesToSave.append(frame)
    
    listOfFramesToSave.reverse()
    
    for i in range(0, len(listOfFramesToSave)):
      print("calculating background, currently at frame:", i)
      frame = listOfFramesToSave[i]
      fgmask = fgbg.apply(frame)
    
    # listOfFramesToSave = []
    # for i in range(0, lastFrame - 1, int((lastFrame - 1) / historyLength if (lastFrame - 1) / historyLength >= 1 else 1))]
      # cap.set(1, lastFrame - 1 - i)
      # print("calculating background, currently at frame:", lastFrame - 1 - i)
      # ret, frame = cap.read()
      # if ret:
        # frame = cv2.resize(frame, (int(frame_width * reduceImageResolutionPercentage), int(frame_height * reduceImageResolutionPercentage)), interpolation = cv2.INTER_AREA)
        # fgmask = fgbg.apply(frame)
        # listOfFramesToSave.append(frame)
    cap.release()
    with open(os.path.join(pathToVideo, videoName + '_BackgroundKNN_' + str(historyLength) + '.pkl'), 'wb') as handle:
      pickle.dump(listOfFramesToSave, handle) #, protocol=pickle.HIGHEST_PROTOCOL)
  
  cap = zzVideoReading.VideoCapture(videoPath)
  
  i = firstFrame
  cap.set(1, firstFrame)
  
  while i < lastFrame:
    
    print(i, lastFrame)
    
    ret, frame = cap.read()
    
    if ret:
    
      frameInitialImage = frame.copy()
      
      if hyperparameters["reduceImageResolutionPercentage"]:
        frame = cv2.resize(frame, (int(frame_width * hyperparameters["reduceImageResolutionPercentage"]), int(frame_height * hyperparameters["reduceImageResolutionPercentage"])), interpolation = cv2.INTER_AREA)
        frameInitialImage = frame.copy()
      
      if i == firstFrame:
        
        WINDOW_NAME = "Click on the center of the head of the animal"
        cvui.init(WINDOW_NAME)
        cv2.moveWindow(WINDOW_NAME, 0,0)
        cvui.imshow(WINDOW_NAME, frame)
        while not(cvui.mouse(cvui.CLICK)):
          cursor = cvui.mouse()
          if cv2.waitKey(20) == 27:
            break
        cv2.destroyAllWindows()
        previousCenterDetectedX = cursor.x
        previousCenterDetectedY = cursor.y
        
        WINDOW_NAME = "Click on the tip of the tail of the same animal"
        cvui.init(WINDOW_NAME)
        cv2.moveWindow(WINDOW_NAME, 0,0)
        cvui.imshow(WINDOW_NAME, frame)
        while not(cvui.mouse(cvui.CLICK)):
          cursor = cvui.mouse()
          if cv2.waitKey(20) == 27:
            break
        cv2.destroyAllWindows()
        tailTipX = cursor.x
        tailTipY = cursor.y
        
        frame2 = frame.copy()
        frame2 = fgbg.apply(frame2)
        ret, frame2 = cv2.threshold(frame2, 0, 255, cv2.THRESH_BINARY)
        frame2 = fillWhiteHoles(frame2)
        frame2 = 255 - frame2
        if iterativelyErodeEachImage:
          frame2 = erodeThenAddWhiteBorders(frame2, kernel)
        else:
          frame2 = addWhiteBorders(frame2)
        contours, hierarchy = cv2.findContours(frame2, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        for contour in contours:
          if cv2.pointPolygonTest(contour, (previousCenterDetectedX, previousCenterDetectedY), False) >= 0:
            animalBodyArea = cv2.contourArea(contour)
            ROIHalfDiam = int(math.sqrt(animalBodyArea) * 4)
        print("animalBodyArea:", animalBodyArea)
        print("ROIHalfDiam:", ROIHalfDiam)
        if showFramesForDebugging:
          cv2.imshow("frame0", frame2)
          cv2.waitKey(0)        
      
      if chooseValueForAnimalBodyArea > 0:
        animalBodyArea = chooseValueForAnimalBodyArea
      
      if ROIHalfDiam == -1:
        raise CustomError("An error occurred")
      
      diffAreaAuthorized = 0.5
      hyperparameters["minAreaBody"] = int(animalBodyArea - animalBodyArea * diffAreaAuthorized)
      hyperparameters["maxAreaBody"] = int(animalBodyArea + animalBodyArea * diffAreaAuthorized)
      hyperparameters["minArea"] = int(animalBodyArea - animalBodyArea * diffAreaAuthorized)
      hyperparameters["maxArea"] = int(animalBodyArea + animalBodyArea * diffAreaAuthorized)
      hyperparameters["headSize"]    = int(math.sqrt(animalBodyArea) * 2)
      hyperparameters["minTailSize"] = int(math.sqrt(animalBodyArea) * 0.5)
      hyperparameters["maxTailSize"] = int(math.sqrt(animalBodyArea) * 4)
      
      xmin = previousCenterDetectedX - ROIHalfDiam if previousCenterDetectedX - ROIHalfDiam > 0 else 0
      xmax = previousCenterDetectedX + ROIHalfDiam if previousCenterDetectedX + ROIHalfDiam < len(frame[0]) else len(frame[0]) - 1
      ymin = previousCenterDetectedY - ROIHalfDiam if previousCenterDetectedY - ROIHalfDiam > 0 else 0
      ymax = previousCenterDetectedY + ROIHalfDiam if previousCenterDetectedY + ROIHalfDiam < len(frame) else len(frame) - 1
      
      previousCenterDetectedXROICoordinates = previousCenterDetectedX - xmin
      previousCenterDetectedYROICoordinates = previousCenterDetectedY - ymin
      
      if showInitialVideo:
        initialImage = frame.copy()
      
      frame = fgbg.apply(frame)
      
      frame = frame[ymin:ymax, xmin:xmax]
      
      ret, frame = cv2.threshold(frame, 0, 255, cv2.THRESH_BINARY)
      
      frame = fillWhiteHoles(frame)
      
      frame = 255 - frame
      
      countNbOfRightArea = 0
      distanceToPreviousCenterDetected = 100000000000000000000
      countNbTries = 0
      newCenterDetectedX_ROICordinates = previousCenterDetectedXROICoordinates
      newCenterDetectedY_ROICordinates = previousCenterDetectedYROICoordinates
      mostLikelyContour = 0
      while countNbOfRightArea == 0 and countNbTries < 10:
        countNbTries = countNbTries + 1
        if iterativelyErodeEachImage:
          frame = erodeThenAddWhiteBorders(frame, kernel)
        else:
          frame = addWhiteBorders(frame)
        if showFramesForDebugging:
          cv2.imshow("frame1", frame)
          cv2.waitKey(0)
        
        nbOfIterations = 1
        while cv2.countNonZero(255 - frame) < animalBodyArea and nbOfIterations < 10:
          print("try " + str(nbOfIterations))
          frame = erodeThenDilateThenAddWhiteBorders(frame, kernel, nbOfIterations)
          nbOfIterations += 1
          if showFramesForDebugging:
            cv2.imshow("frame" + str(nbOfIterations), frame)
            cv2.waitKey(0)
        
        if nbOfIterations > 1:
          dist2t = fgbg.getDist2Threshold()
          fgbg.setDist2Threshold(dist2t - dist2t * 0.1)
          print("dist2t set to a smaller value:", dist2t - dist2t * 0.1)
        else:
          if cv2.countNonZero(255 - frame) > 1.5 * animalBodyArea:
            dist2t = fgbg.getDist2Threshold()
            fgbg.setDist2Threshold(dist2t + dist2t * 0.1)
            print("dist2t set to a bigger value:", dist2t + dist2t * 0.1)
        
        contours, hierarchy = cv2.findContours(frame, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        for contour in contours:
          contourArea = cv2.contourArea(contour)
          if contourArea > hyperparameters["minAreaBody"] and contourArea < hyperparameters["maxAreaBody"]:
            countNbOfRightArea = countNbOfRightArea + 1
            M = cv2.moments(contour)
            cx = int(M['m10']/M['m00'])
            cy = int(M['m01']/M['m00'])
            [cx, cy] = reajustCenterOfMassIfNecessary(contour, cx, cy, len(frame[0]), len(frame))
            if math.sqrt((previousCenterDetectedXROICoordinates - cx)**2 + (previousCenterDetectedYROICoordinates - cy)**2) < distanceToPreviousCenterDetected:
              newCenterDetectedX_ROICordinates = cx
              newCenterDetectedY_ROICordinates = cy
              mostLikelyContour = contour
        
        if not(iterativelyErodeEachImage):
          countNbTries = 10
        # print("First: newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates:", newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates)
        
        if type(mostLikelyContour) != int:
          
          cntXmin = 100000000000000
          cntYmin = 100000000000000
          cntXmax = -1
          cntYmax = -1
          
          for point in mostLikelyContour:
            if point[0][0] < cntXmin:
              cntXmin = point[0][0]
            if point[0][1] < cntYmin:
              cntYmin = point[0][1]
            if point[0][0] > cntXmax:
              cntXmax = point[0][0]
            if point[0][1] > cntYmax:
              cntYmax = point[0][1]
        
            cntXmin = cntXmin - 10
            cntYmin = cntYmin - 10
            cntXmax = cntXmax + 10
            cntYmax = cntYmax + 10
          
          if False:
            
            print("This is False")
            # for point in mostLikelyContour:
              # point[0][0] = point[0][0] - cntXmin
              # point[0][1] = point[0][1] - cntYmin
            
            # blank = np.zeros((cntYmax - cntYmin, cntXmax - cntXmin, 3), np.uint8)
            # blank = 255 - blank
            
            # blank = cv2.fillPoly(blank, pts =[mostLikelyContour], color=(0, 0, 0))
            
            # blank = cv2.dilate(blank, kernel, iterations=1)
            # blankIni = blank.copy()
            
            # nbDilationCount = 0
            # blankPreviousIteration = blank
            
            # while cv2.countNonZero(cv2.cvtColor(255 - blank, cv2.COLOR_BGR2GRAY)) != 0 and nbDilationCount < 10:
              # blankPreviousIteration = cv2.cvtColor(blank.copy(), cv2.COLOR_BGR2GRAY)
              # blank = cv2.dilate(blank, kernel, iterations=1)
              # nbDilationCount = nbDilationCount + 1
            
            # contours, hierarchy = cv2.findContours(blankPreviousIteration, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
            # largestContourArea = 0
            # for contour in contours:
              # contourArea = cv2.contourArea(contour)
              # if contourArea > largestContourArea:
                # largestContourArea = contourArea
                # M = cv2.moments(contour)
                # x = int(M['m10']/M['m00'])
                # y = int(M['m01']/M['m00'])
                # if False:
                  # [newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates] = reajustCenterOfMassIfNecessary(contour, x, y, len(blankPreviousIteration[0]), len(blankPreviousIteration))
                # newCenterDetectedX_ROICordinates = x
                # newCenterDetectedY_ROICordinates = y
          
          else:
              
            cntXmin = 10
            cntYmin = 10
            
            blank = np.zeros((xmax - xmin, ymax - ymin, 3), np.uint8)
            blank = 255 - blank
            
            blank = cv2.fillPoly(blank, pts =[mostLikelyContour], color=(0, 0, 0))
            
            blank = cv2.dilate(blank, kernel, iterations=1)
            blankIni = blank.copy()
            
            nbDilationCount = 0
            blankPreviousIteration = blank
            
            while cv2.countNonZero(cv2.cvtColor(255 - blank, cv2.COLOR_BGR2GRAY)) != 0 and nbDilationCount < 10:
              blankPreviousIteration = cv2.cvtColor(blank.copy(), cv2.COLOR_BGR2GRAY)
              blank = cv2.dilate(blank, kernel, iterations=1)
              nbDilationCount = nbDilationCount + 1
            
            if type(blankPreviousIteration) == np.ndarray and len(blankPreviousIteration) and len(blankPreviousIteration[0]) and type(blankPreviousIteration[0][0]) == np.uint8:
              contours, hierarchy = cv2.findContours(blankPreviousIteration, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
              largestContourArea = 0
              for contour in contours:
                contourArea = cv2.contourArea(contour)
                if contourArea > largestContourArea and contourArea < (len(blankPreviousIteration)*len(blankPreviousIteration[0])) * 0.8:
                  largestContourArea = contourArea
                  M = cv2.moments(contour)
                  x = int(M['m10']/M['m00'])
                  y = int(M['m01']/M['m00'])
                  # if False:
                    # [newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates] = reajustCenterOfMassIfNecessary(contour, x, y, len(blankPreviousIteration[0]), len(blankPreviousIteration))
                  newCenterDetectedX_ROICordinates = x
                  newCenterDetectedY_ROICordinates = y
      
      # print("Second: newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates:", newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates)
      
      trackingHeadTailAllAnimals[0, i-firstFrame][0][0] = newCenterDetectedX_ROICordinates
      trackingHeadTailAllAnimals[0, i-firstFrame][0][1] = newCenterDetectedY_ROICordinates
      
      if hyperparameters["trackTail"] == 1:
        if type(blankIni) == np.ndarray and len(blankIni) and len(blankIni[0]):
          if type(blankIni[0][0]) != np.uint8:
            blankIni = cv2.cvtColor(blankIni, cv2.COLOR_BGR2GRAY)
          for animalId in range(0, hyperparameters["nbAnimalsPerWell"]):
            [trackingHeadTailAllAnimals, trackingHeadingAllAnimals] = tailTracking(animalId, i, firstFrame, videoPath, blankIni, hyperparameters, blankIni, nbTailPoints, blankIni, 0, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, 0, 0, 0, blankIni, 0, wellNumber)
        else:
          print("problem")
      
      for j in range(0, len(trackingHeadTailAllAnimals[0, i-firstFrame])):
        trackingHeadTailAllAnimals[0, i-firstFrame][j][0] += xmin + cntXmin - 10
        trackingHeadTailAllAnimals[0, i-firstFrame][j][1] += ymin + cntYmin - 10
      
      debugTracking(nbTailPoints, i, firstFrame, trackingHeadTailAllAnimals, trackingHeadingAllAnimals, initialImage, hyperparameters)
      # if showInitialVideo:
        # initialImage = cv2.circle(initialImage, (newCenterDetectedX_ROICordinates + xmin + cntXmin - 10, newCenterDetectedY_ROICordinates + ymin + cntYmin - 10), 3, (255, 0, 0), 2)
        # cv2.imshow("frame", initialImage)
        # cv2.waitKey(0)
      # else:
        # frame = cv2.circle(frame, (newCenterDetectedX_ROICordinates, newCenterDetectedY_ROICordinates), 3, (255, 0, 0), 2)
        # cv2.imshow("frame", frame)
        # cv2.waitKey(0)
      
      previousCenterDetectedX = newCenterDetectedX_ROICordinates + xmin + cntXmin - 10
      previousCenterDetectedY = newCenterDetectedY_ROICordinates + ymin + cntYmin - 10
    
    i = i + 1

  cv2.destroyAllWindows()
  
  return [trackingHeadTailAllAnimals, trackingHeadingAllAnimals, trackingEyesAllAnimals]



import numpy as np
from scipy.interpolate import UnivariateSpline

def getTailAngleRecalculated2(curbout, nbFramesTakenIntoAccount, numberOfBendsIncludedForMaxDetect):

  fin = curbout["BoutEnd"] - curbout["BoutStart"] + 1
  
  numps = 3
  
  headx = curbout["HeadX"]
  heady = curbout["HeadY"]
  tailx = curbout["TailX_VideoReferential"]
  taily = curbout["TailY_VideoReferential"]
  
  tailangles_arr = np.zeros((nbFramesTakenIntoAccount,7))
  for i in range(min(len(headx),tailangles_arr.shape[0])):
    if len(taily[i]) > 3 and len(tailx[i]) > 3:
      ang = np.arctan2(heady[i] - taily[i][-3],headx[i] - tailx[i][-3])*180/np.pi
      for j in range(tailangles_arr.shape[1]):
        ang2 = np.arctan2(heady[i] - taily[i][j],headx[i] - tailx[i][j])*180/np.pi
        delang = ang2 - ang
        if np.abs(delang) < 180:
          tailangles_arr[i,j] = delang
        elif delang > 180:
          tailangles_arr[i,j] = delang - 360
        elif delang < -180:
          tailangles_arr[i,j] = 360 + delang
    else:
      for j in range(tailangles_arr.shape[1]):
        tailangles_arr[i,j] = 0
    
  tailangles_arr = tailangles_arr.flatten()
  
  return tailangles_arr



from zebrazoom.code.preprocessImage import preprocessImage
import numpy as np
import cv2

def getImageSequential(cap, videoPath, frameNumber, wellNumber, wellPositions, hyperparameters):
  
  minPixelDiffForBackExtract = hyperparameters["minPixelDiffForBackExtract"]
  debug = 0
  
  xtop = wellPositions[wellNumber]['topLeftX']
  ytop = wellPositions[wellNumber]['topLeftY']
  lenX = wellPositions[wellNumber]['lengthX']
  lenY = wellPositions[wellNumber]['lengthY']
  
  ret, frame = cap.read()
  
  while not(ret):
    print("WARNING: was not able to extract the frame", str(frameNumber),"in 'getImageSequential'")
    frameNumber = frameNumber - 1
    cap.set(1, frameNumber)
    ret, frame = cap.read()
    
  if hyperparameters["invertBlackWhiteOnImages"]:
    frame = 255 - frame
  
  if hyperparameters["imagePreProcessMethod"]:
    frame = preprocessImage(frame, hyperparameters)
  
  grey = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
  curFrame = grey[ytop:ytop+lenY, xtop:xtop+lenX]
  
  if (debug):
    cv2.imshow('Frame', curFrame)
    cv2.waitKey(0)
    
  return curFrame



from zebrazoom.code.findWells import findWells
from zebrazoom.code.getBackground import getBackground
from zebrazoom.code.getImage.getForegroundImage import getForegroundImage
from zebrazoom.code.trackingFolder.tracking import tracking
from zebrazoom.code.extractParameters import extractParameters
from zebrazoom.code.createSuperStruct import createSuperStruct
from zebrazoom.code.createValidationVideo import createValidationVideo
from zebrazoom.code.getHyperparameters import getHyperparameters
import zebrazoom.code.popUpAlgoFollow as popUpAlgoFollow
from zebrazoom.code.dataPostProcessing.dataPostProcessing import dataPostProcessing
from zebrazoom.code.fasterMultiprocessing import fasterMultiprocessing

import sys
import pickle
import os
import shutil
import time
import cv2
import zebrazoom.videoFormatConversion.zzVideoReading as zzVideoReading
import json
import subprocess
import glob

try:
  from PyQt6.QtWidgets import QApplication
except ImportError:
  from PyQt5.QtWidgets import QApplication

from zebrazoom.code.vars import getGlobalVariables
globalVariables = getGlobalVariables()

output = 0

# Does the tracking and then the extraction of parameters
def getParametersForWell(videoPath,background,wellNumber,wellPositions,output,previouslyAcquiredTrackingDataForDebug,hyperparameters, videoName, dlModel):
  if hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "noDebug":
    # Normal execution process
    trackingData = tracking(videoPath,background,wellNumber,wellPositions,hyperparameters, videoName, dlModel)
    parameters = extractParameters(trackingData, wellNumber, hyperparameters, videoPath, wellPositions, background)
    output.put([wellNumber,parameters,[]])
  elif hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "justSaveTrackData":
    # Extracing tracking data, saving it, and that's it
    trackingData = tracking(videoPath,background,wellNumber,wellPositions,hyperparameters, videoName, dlModel)
    output.put([wellNumber,[],trackingData])
  elif hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "saveTrackDataAndExtractParam":
    # Extracing tracking data, saving it, and continuing normal execution
    trackingData = tracking(videoPath,background,wellNumber,wellPositions,hyperparameters, videoName, dlModel)
    parameters = extractParameters(trackingData, wellNumber, hyperparameters, videoPath, wellPositions, background)
    output.put([wellNumber,parameters,trackingData])
  else: # hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "justExtractParamFromPreviousTrackData"
    # Reloading previously tracked data and using it to extract parameters
    trackingData = previouslyAcquiredTrackingDataForDebug[wellNumber]
    parameters = extractParameters(trackingData, wellNumber, hyperparameters, videoPath, wellPositions, background)
    output.put([wellNumber,parameters,[]])


# Getting parameters
if len(sys.argv) > 2:
  pathToVideo = sys.argv[1]
  videoName   = sys.argv[2]
  videoExt    = sys.argv[3] # Ext means extension (avi, mp4, etc...)
  configFile  = sys.argv[4]
  argv        = sys.argv
else:
  configFile  = ""


def mainZZ(pathToVideo, videoName, videoExt, configFile, argv):
  
  videoNameWithExt = videoName + '.' + videoExt
  previouslyAcquiredTrackingDataForDebug = []

  # Checking that path and video exists
  if not(os.path.exists(os.path.join(pathToVideo, videoNameWithExt))):
    print("Path or video name is incorrect for", os.path.join(pathToVideo, videoNameWithExt))
    return 0

  # Getting hyperparameters
  [hyperparameters, configFile] = getHyperparameters(configFile, videoNameWithExt, os.path.join(pathToVideo, videoNameWithExt), argv)
  
  if hyperparameters["trackingDL"]:
    import torch.multiprocessing as mp
  else:
    import multiprocessing as mp
  if globalVariables["mac"] or hyperparameters["trackingDL"]:
    mp.set_start_method('spawn', force=True)
  if hyperparameters["trackingDL"]:
    from torch.multiprocessing import Process
  else:
    from multiprocessing import Process

  output = mp.Queue()
  
  # Checking first frame and last frame value
  cap   = zzVideoReading.VideoCapture(os.path.join(pathToVideo, videoNameWithExt))
  nbFrames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
  cap.release()
  if hyperparameters["firstFrame"] < 0:
    print("Error for video " + videoName + ": The parameter 'firstFrame' in your configuration file is too small" + " (firstFrame value is " + str(hyperparameters["firstFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
    raise NameError("Error for video " + videoName + ": The parameter 'firstFrame' in your configuration file is too small" + " (firstFrame value is " + str(hyperparameters["firstFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
  if hyperparameters["firstFrame"] > nbFrames:
    print("Error for video " + videoName + ": The parameter 'firstFrame' in your configuration file is too big" + " (firstFrame value is " + str(hyperparameters["firstFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
    raise NameError("Error for video " + videoName + ": The parameter 'firstFrame' in your configuration file is too big" + " (firstFrame value is " + str(hyperparameters["firstFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
  if (hyperparameters["lastFrame"] < 0) or (hyperparameters["lastFrame"] <= hyperparameters["firstFrame"]):
    print("Error for video " + videoName + ": The parameter 'lastFrame' in your configuration file is too small" + " (lastFrame value is " + str(hyperparameters["lastFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
    raise NameError("Error for video " + videoName + ": The parameter 'lastFrame' in your configuration file is too small" + " (lastFrame value is " + str(hyperparameters["lastFrame"]) + ", number of frames in the video is " + str(nbFrames) + ")")
  if hyperparameters["lastFrame"] > nbFrames:
    print("Warning: The parameter 'lastFrame' in your configuration file is too big so we adjusted it to the value:", nbFrames-2, "it was originally set to", hyperparameters["lastFrame"])
    hyperparameters["lastFrame"] = nbFrames - 2
    # raise NameError("Error: The parameter 'lastFrame' in your configuration file is too big")
  
  # Setting output folder
  outputFolderVideo = os.path.join(hyperparameters["outputFolder"], videoName)
  
  # Launching GUI algoFollower if necessary
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.initialise()
    popUpAlgoFollow.prepend("starting ZebraZoom analysis on " + videoName)

  if hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "justExtractParamFromPreviousTrackData":
    # Reloading previously extracted tracking data if debugging option selected
    outfile = open(os.path.join(outputFolderVideo, 'intermediaryTracking.txt'),'rb')
    previouslyAcquiredTrackingDataForDebug = pickle.load(outfile)
    outfile.close()
  else:
    # Creating output folder
    if not(hyperparameters["reloadWellPositions"]) and not(hyperparameters["reloadBackground"]) and not(os.path.exists(os.path.join(outputFolderVideo, 'intermediaryWellPositionReloadNoMatterWhat.txt'))) and not(hyperparameters["dontDeleteOutputFolderIfAlreadyExist"]):
      if os.path.exists(outputFolderVideo):
        if glob.glob(os.path.join(outputFolderVideo, 'results_*.txt')) and glob.glob(os.path.join(outputFolderVideo, '*.avi')):
          pastNumbersTaken = 1
          while os.path.exists(outputFolderVideo + '_PastTracking_' + str(pastNumbersTaken)) and pastNumbersTaken < 99:
            pastNumbersTaken += 1
          shutil.move(outputFolderVideo, outputFolderVideo + '_PastTracking_' + str(pastNumbersTaken))
        else:
          shutil.rmtree(outputFolderVideo)
      if hyperparameters["tryCreatingFolderUntilSuccess"]:
        while True:
          try:
            os.mkdir(outputFolderVideo)
            break
          except OSError as e:
            print("waiting inside except")
            time.sleep(0.1)
          else:
            print("waiting")
            time.sleep(0.1)
      else:
        try:
          os.mkdir(outputFolderVideo)
        except OSError as e:
          time.sleep(0.1)
        else:
          time.sleep(0.1)
  
  # Saving the configuration file used
  with open(os.path.join(outputFolderVideo, 'configUsed.json'), 'w') as outfile:
    json.dump(configFile, outfile)
  

  # Getting well positions
  if hyperparameters["headEmbeded"]:
    wellPositions = [{"topLeftX":0, "topLeftY":0, "lengthX": hyperparameters["videoWidth"], "lengthY": hyperparameters["videoHeight"]}]
  else:
    print("start find wells")
    if hyperparameters["saveWellPositionsToBeReloadedNoMatterWhat"]:
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryWellPositionReloadNoMatterWhat.txt'),'wb')
      wellPositions = findWells(os.path.join(pathToVideo, videoNameWithExt), hyperparameters)
      pickle.dump(wellPositions,outfile)
    elif os.path.exists(os.path.join(outputFolderVideo, 'intermediaryWellPositionReloadNoMatterWhat.txt')):
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryWellPositionReloadNoMatterWhat.txt'), 'rb')
      wellPositions = pickle.load(outfile)
    elif hyperparameters["reloadWellPositions"]:
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryWellPosition.txt'), 'rb')
      wellPositions = pickle.load(outfile)
    elif hyperparameters["reloadWellPositionsFromFileInZZoutputIfItExistSaveInItOtherwise"] and os.path.exists(os.path.join(hyperparameters["outputFolder"], 'wellPosition.txt')):
      outfile = open(os.path.join(hyperparameters["outputFolder"], 'wellPosition.txt'), 'rb')
      wellPositions = pickle.load(outfile)
    else:
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryWellPosition.txt'),'wb')
      wellPositions = findWells(os.path.join(pathToVideo, videoNameWithExt), hyperparameters)
      pickle.dump(wellPositions,outfile)
      if hyperparameters["reloadWellPositionsFromFileInZZoutputIfItExistSaveInItOtherwise"]:
        outfile2 = open(os.path.join(hyperparameters["outputFolder"], 'wellPosition.txt'), 'wb')
        pickle.dump(wellPositions, outfile2)
        outfile2.close()
    outfile.close()
    app = QApplication.instance()
    if hasattr(app, "wellPositions"):
      if wellPositions is None:
        return
      app.wellPositions[:] = [(position['topLeftX'], position['topLeftY'], position['lengthX'], position['lengthY'])
                              for idx, position in enumerate(wellPositions)]
      if hyperparameters["wellsAreRectangles"] or len(hyperparameters["oneWellManuallyChosenTopLeft"]) or int(hyperparameters["multipleROIsDefinedDuringExecution"]) or hyperparameters["noWellDetection"] or hyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"]:
        app.wellShape = 'rectangle'
      else:
        app.wellShape = 'circle'

  if int(hyperparameters["exitAfterWellsDetection"]):
    print("exitAfterWellsDetection")
    if hyperparameters["popUpAlgoFollow"]:
      popUpAlgoFollow.prepend("ZebraZoom Analysis finished for " + videoName)
    raise ValueError
    
  # Getting background
  if hyperparameters["backgroundSubtractorKNN"] or (hyperparameters["headEmbeded"] and hyperparameters["headEmbededRemoveBack"] == 0 and hyperparameters["headEmbededAutoSet_BackgroundExtractionOption"] == 0 and hyperparameters["adjustHeadEmbededTracking"] == 0) or hyperparameters["trackingDL"] or hyperparameters["fishTailTrackingDifficultBackground"]:
    background = []
  else:
    print("start get background")
    if hyperparameters["reloadBackground"]:
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryBackground.txt'),'rb')
      background = pickle.load(outfile)
      print("Background Reloaded")
    else:
      outfile = open(os.path.join(outputFolderVideo, 'intermediaryBackground.txt'),'wb')
      background = getBackground(os.path.join(pathToVideo, videoNameWithExt), hyperparameters)
      pickle.dump(background, outfile)
      cv2.imwrite(os.path.join(outputFolderVideo, 'background.png'), background)
    outfile.close()
  if hyperparameters["exitAfterBackgroundExtraction"]:
    print("exitAfterBackgroundExtraction")
    raise ValueError
  
  # Reloading DL model for tracking with DL
  if hyperparameters["trackingDL"]:
    from zebrazoom.code.deepLearningFunctions.loadDLmodel import loadDLmodel
    dlModel = loadDLmodel(hyperparameters["trackingDL"])
  else:
    dlModel = 0
  
  # Tracking and extraction of parameters
  if hyperparameters["fasterMultiprocessing"] == 1:
    processes = -1
    output2 = fasterMultiprocessing(os.path.join(pathToVideo, videoNameWithExt), background, wellPositions, [], hyperparameters, videoName)
  else:
    if globalVariables["noMultiprocessing"] == 0:
      if hyperparameters["onlyTrackThisOneWell"] == -1:
        # for all wells, in parallel
        processes = []
        for wellNumber in range(0,hyperparameters["nbWells"]):
          p = Process(target=getParametersForWell, args=(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel))
          p.start()
          processes.append(p)
      else:
        # for just one well
        processes = [1]
        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
    else:
      if hyperparameters["onlyTrackThisOneWell"] == -1:
        processes = [1 for i in range(0, hyperparameters["nbWells"])]
        for wellNumber in range(0,hyperparameters["nbWells"]):
          getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, wellNumber, wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
      else:
        processes = [1]
        getParametersForWell(os.path.join(pathToVideo, videoNameWithExt), background, hyperparameters["onlyTrackThisOneWell"], wellPositions, output, previouslyAcquiredTrackingDataForDebug, hyperparameters, videoName, dlModel)
  
  # Sorting wells after the end of the parallelized calls end
  if processes != -1:
    dataPerWellUnsorted = [output.get() for p in processes]
  else:
    dataPerWellUnsorted = output2
  paramDataPerWell = [[]] * (hyperparameters["nbWells"])
  trackingDataPerWell = [[]] * (hyperparameters["nbWells"])
  for data in dataPerWellUnsorted:
    paramDataPerWell[data[0]]    = data[1]
    trackingDataPerWell[data[0]] = data[2]
  if processes != -1 and hyperparameters["onlyTrackThisOneWell"] == -1 and globalVariables["noMultiprocessing"] == 0:
    for p in processes:
      p.join()
  if (hyperparameters["freqAlgoPosFollow"] != 0):
    print("processes joined")

  if (hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "saveTrackDataAndExtractParam") or (hyperparameters["debugPauseBetweenTrackAndParamExtract"] == "justSaveTrackData"):
    # saving tracking results for future uses
    outfile = open(os.path.join(outputFolderVideo, 'intermediaryTracking.txt'),'wb')
    pickle.dump(trackingDataPerWell,outfile)
    outfile.close()
    if (hyperparameters["freqAlgoPosFollow"] != 0):
      print("intermediary results saved")
    
  if hyperparameters["debugPauseBetweenTrackAndParamExtract"] != "justSaveTrackData":
    # Creating super structure
    superStruct = createSuperStruct(paramDataPerWell, wellPositions, hyperparameters)
  
    # Creating validation video
    if hyperparameters["copyOriginalVideoToOutputFolderForValidation"]:
      shutil.copyfile(os.path.join(pathToVideo, videoNameWithExt), os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'originalVideoWithoutAnyTrackingDisplayed_pleaseUseTheGUIToVisualizeTrackingPoints.avi'))
    else:
      if hyperparameters["createValidationVideo"]:
        infoFrame = createValidationVideo(os.path.join(pathToVideo, videoNameWithExt), superStruct, hyperparameters)
    
    # Various post-processing options depending on configuration file choices
    superStruct = dataPostProcessing(outputFolderVideo, superStruct, hyperparameters, videoName, videoExt)
    
    path = os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'results_' + hyperparameters["videoName"] + '.txt')
    print("createSuperStruct:", path)
    with open(path, 'w') as outfile:
      json.dump(superStruct, outfile)
    if hyperparameters["saveSuperStructToMatlab"]:
      from scipy.io import savemat
      matlabPath = os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'results_' + hyperparameters["videoName"] + '.mat')
      videoDataResults2 = {}
      videoDataResults2['videoDataResults'] = superStruct
      savemat(matlabPath, videoDataResults2)
  
    
  try:
    with open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w') as fp:
      subprocess.run(["pip", "show",  "zebrazoom"], stdout=fp)
  except:
    fileVersionUsed = open(os.path.join(os.path.join(hyperparameters["outputFolder"], hyperparameters["videoName"]), 'ZebraZoomVersionUsed.txt'), 'w')
    fileVersionUsed.write("Was not able to retrive the version number used.")
    fileVersionUsed.close()
  
  # Copying output result folder in another folder
  if len(hyperparameters["additionalOutputFolder"]):
    if os.path.isdir(hyperparameters["additionalOutputFolder"]):
      if hyperparameters["additionalOutputFolderOverwriteIfAlreadyExist"]:
        shutil.rmtree(hyperparameters["additionalOutputFolder"])
        while True:
          try:
            shutil.copytree(outputFolderVideo, hyperparameters["additionalOutputFolder"])
            break
          except OSError as e:
            print("waiting inside except")
            time.sleep(0.1)
          else:
            print("waiting")
            time.sleep(0.1)
      else:
        print("The path " + hyperparameters["additionalOutputFolder"] + " already exists. New folder not created. If you want the folder to be overwritten in such situation in future executions, set the parameter 'additionalOutputFolderOverwriteIfAlreadyExist' to 1 in your configuration file.")
    else:
      shutil.copytree(outputFolderVideo, hyperparameters["additionalOutputFolder"])
  
  
  if hyperparameters["popUpAlgoFollow"]:
    popUpAlgoFollow.prepend("ZebraZoom Analysis finished for " + videoName)
    # popUpAlgoFollow.prepend("")
    # if hyperparameters["closePopUpWindowAtTheEnd"]:
      # popUpAlgoFollow.prepend("ZebraZoom Analysis all finished")




from pathlib import Path
import numpy as np
import json
import cv2
import math
import cvui
from zebrazoom.code.vars import getGlobalVariables
import json
import os
import subprocess
import sys
import pandas as pd
globalVariables = getGlobalVariables()

try:
  from PyQt6.QtWidgets import QFileDialog
except ImportError:
  from PyQt5.QtWidgets import QFileDialog

from zebrazoom.dataAnalysis.datasetcreation.createDataFrame import createDataFrame
from zebrazoom.dataAnalysis.dataanalysis.populationComparaison import populationComparaison
from zebrazoom.dataAnalysis.dataanalysis.applyClustering import applyClustering

from zebrazoom.dataAnalysis.datasetcreation.generatePklDataFileForVideo import generatePklDataFileForVideo


def openAnalysisFolder(self, homeDirectory, specificDirectory):
  dir_path = os.path.join(os.path.join(homeDirectory,'dataAnalysis'), specificDirectory)
  if sys.platform == "win32":
    os.startfile(dir_path)
  else:
    opener ="open" if sys.platform == "darwin" else "xdg-open"
    subprocess.call([opener, dir_path])


def populationComparison(self, controller, TailTrackingParameters=0, saveInMatlabFormat=0, saveRawData=0, forcePandasRecreation=0, minNbBendForBoutDetect=3, discard=0, keep=1, frameStepForDistanceCalculation='4'):
  
  if len(frameStepForDistanceCalculation) == 0:
    frameStepForDistanceCalculation = '4'
  
  if discard == 0 and keep == 0:
    keep = 1

  if len(minNbBendForBoutDetect) == 0:
    minNbBendForBoutDetect = 3

  cur_dir_path = os.path.dirname(os.path.realpath(__file__))
  cur_dir_path = Path(cur_dir_path)
  cur_dir_path = cur_dir_path.parent.parent

  if len(controller.ZZoutputLocation) == 0:
    ZZoutputLocation = os.path.join(cur_dir_path, 'ZZoutput')
  else:
    ZZoutputLocation = controller.ZZoutputLocation

  # Creating the dataframe

  dataframeOptions = {
    'pathToExcelFile'                   : self.experimentOrganizationExcelFileAndFolder, #os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'experimentOrganizationExcel/')),
    'fileExtension'                     : '.' + self.experimentOrganizationExcel.split(".")[1],
    'resFolder'                         : os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'data')),
    'nameOfFile'                        : self.experimentOrganizationExcel.split(".")[0],
    'smoothingFactorDynaParam'          : 0,   # 0.001
    'nbFramesTakenIntoAccount'          : 0,
    'numberOfBendsIncludedForMaxDetect' : -1,
    'minNbBendForBoutDetect'            : int(minNbBendForBoutDetect),
    'keepSpeedDistDurWhenLowNbBends'    : int(keep),
    'defaultZZoutputFolderPath'         : ZZoutputLocation,
    'computeTailAngleParamForCluster'   : False,
    'computeMassCenterParamForCluster'  : False,
    'tailAngleKinematicParameterCalculation'    : TailTrackingParameters,
    'saveRawDataInAllBoutsSuperStructure'       : saveRawData,
    'saveAllBoutsSuperStructuresInMatlabFormat' : saveInMatlabFormat,
    'frameStepForDistanceCalculation'           : frameStepForDistanceCalculation
  }
  
  generatePklDataFileForVideo(os.path.join(self.experimentOrganizationExcelFileAndFolder, self.experimentOrganizationExcel), ZZoutputLocation, frameStepForDistanceCalculation, forcePandasRecreation)
  
  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions, "", forcePandasRecreation, [])
  
  # Plotting for the different conditions
  nameOfFile = dataframeOptions['nameOfFile']
  resFolder  = dataframeOptions['resFolder']

  # Mixing up all the bouts
  populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 0, True)

  populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 0, False)

  # First median per well for each kinematic parameter
  populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 1, True)

  populationComparaison(nameOfFile, resFolder, globParam, conditions, genotypes, os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'resultsKinematic')), 1, False)

  controller.show_frame("AnalysisOutputFolderPopulation")


def boutClustering(self, controller, nbClustersToFind, FreelySwimming, HeadEmbeded, minNbBendForBoutDetect=3, nbVideosToSave=0, modelUsedForClustering=0, removeOutliers=False, frameStepForDistanceCalculation='4', removeBoutsContainingNanValuesInParametersUsedForClustering=True, forcePandasRecreation=0):

  if len(frameStepForDistanceCalculation) == 0:
    frameStepForDistanceCalculation = '4'
  
  if modelUsedForClustering == 0:
    modelUsedForClustering = 'KMeans'
  else:
    modelUsedForClustering = 'GaussianMixture'

  if len(minNbBendForBoutDetect) == 0:
    minNbBendForBoutDetect = 3
  else:
    minNbBendForBoutDetect = int(minNbBendForBoutDetect)

  if len(nbVideosToSave) == 0:
    nbVideosToSave = 0
  else:
    nbVideosToSave = int(nbVideosToSave)

  videoSaveFirstTenBouts = True if nbVideosToSave else False

  cur_dir_path = os.path.dirname(os.path.realpath(__file__))
  cur_dir_path = Path(cur_dir_path)
  cur_dir_path = cur_dir_path.parent.parent

  if len(controller.ZZoutputLocation) == 0:
    ZZoutputLocation = os.path.join(cur_dir_path, 'ZZoutput')
  else:
    ZZoutputLocation = controller.ZZoutputLocation

  # Creating the dataframe on which the clustering will be applied
  dataframeOptions = {
    'pathToExcelFile'                   : self.experimentOrganizationExcelFileAndFolder, # os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'experimentOrganizationExcel')),
    'fileExtension'                     : '.' + self.experimentOrganizationExcel.split(".")[1],
    'resFolder'                         : os.path.join(cur_dir_path, os.path.join('dataAnalysis', 'data')),
    'nameOfFile'                        : self.experimentOrganizationExcel.split(".")[0],
    'smoothingFactorDynaParam'          : 0,   # 0.001
    'nbFramesTakenIntoAccount'          : -1, #28,
    'numberOfBendsIncludedForMaxDetect' : -1,
    'minNbBendForBoutDetect'            : minNbBendForBoutDetect,
    'defaultZZoutputFolderPath'         : ZZoutputLocation,
    'tailAngleKinematicParameterCalculation' : 1,
    'getTailAngleSignMultNormalized'    : 1,
    'computeTailAngleParamForCluster'   : True,
    'computeMassCenterParamForCluster'  : False
  }
  if int(FreelySwimming):
    dataframeOptions['computeMassCenterParamForCluster'] = True
    
  generatePklDataFileForVideo(os.path.join(self.experimentOrganizationExcelFileAndFolder, self.experimentOrganizationExcel), ZZoutputLocation, frameStepForDistanceCalculation, forcePandasRecreation)
    
  [conditions, genotypes, nbFramesTakenIntoAccount, globParam] = createDataFrame(dataframeOptions, "", forcePandasRecreation, [])
  # Applying the clustering on this dataframe
  clusteringOptions = {
    'analyzeAllWellsAtTheSameTime' : 0, # put this to 1 for head-embedded videos, and to 0 for multi-well videos
    'pathToVideos' : ZZoutputLocation,
    'nbCluster' : int(nbClustersToFind),
    #'nbPcaComponents' : 30,
    'nbFramesTakenIntoAccount' : nbFramesTakenIntoAccount,
    'scaleGraphs' : True,
    'showFigures' : False,
    'useFreqAmpAsym' : False,
    'useAngles' : False,
    'useAnglesSpeedHeadingDisp' : False,
    'useAnglesSpeedHeading' : False,
    'useAnglesSpeed' : False,
    'useAnglesHeading' : False,
    'useAnglesHeadingDisp' : False,
    'useFreqAmpAsymSpeedHeadingDisp' : False,
    'videoSaveFirstTenBouts' : videoSaveFirstTenBouts,
    'globalParametersCalculations' : True,
    'nbVideosToSave' : nbVideosToSave,
    'resFolder' : os.path.join(os.path.join(cur_dir_path, 'dataAnalysis'),'data/'),
    'nameOfFile' : self.experimentOrganizationExcel.split(".")[0],
    'modelUsedForClustering' : modelUsedForClustering,
    'removeOutliers'         : removeOutliers,
    'removeBoutsContainingNanValuesInParametersUsedForClustering' : removeBoutsContainingNanValuesInParametersUsedForClustering
  }
  if int(FreelySwimming):
    clusteringOptions['useAnglesSpeedHeading'] = True
    # clusteringOptions['useAngleAnd3GlobalParameters'] = True
    # clusteringOptions['useFreqAmpAsym'] = True
  if int(HeadEmbeded):
    clusteringOptions['useAngles'] = True
  # Applies the clustering
  [allBouts, classifier] = applyClustering(clusteringOptions, 0, os.path.join(os.path.join(cur_dir_path, 'dataAnalysis'),'resultsClustering/'), self.ZZoutputLocation)
  # Saves the classifier
  controller.show_frame("AnalysisOutputFolderClustering")

